/**
 * Copyright 2005 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
package org.apache.nutch.indexer.geo;

import java.util.logging.Logger;
import org.apache.nutch.util.LogFormatter;
import org.apache.nutch.util.NutchConf;

import java.lang.Math;

import java.text.*;

/** Calculate positions on earth */
public class Domain2Position  {

  public static final Logger LOG
    = LogFormatter.getLogger(Domain2Position.class.getName());

  private static final int radiusEarth = 6367000; // in meters
  private static float FloatPI = (float)java.lang.Math.PI;

  public static final int STEP =
    new NutchConf().getInt("geoPosition.step", 1000);

  private static int EarthStep;


  private static int maxAllowed;
  private static int minAllowed;
  private static String positionLength;

  //initialize min-,maxAllowd and positionLength at begin
  static { 
      maxAllowed = 1;
      positionLength = "0";
      EarthStep = radiusEarth / STEP;

      for (int i=0;i < new Integer(EarthStep).toString().length();i++){
          positionLength += "0";
          maxAllowed *= 10;          
      }
      minAllowed = maxAllowed * -1;
      maxAllowed--;
  }

  private float north, east; 

  public Domain2Position(float NorthDeg, float EastDeg) { 
      north = NorthDeg / 180 * FloatPI;
      east  = EastDeg  / 180 * FloatPI;
  }

  public void setNorth(float setNorth) {
      north = setNorth / 180 * FloatPI;
  }

  public void setEast(float setEast) {
      east = setEast / 180 * FloatPI;
  }    

  public float getNorth() {
      return north;
  }

  public float getEast() {
      return east;
  }  

  public int getX() {
      return (int)Math.round(EarthStep * Math.sin(north+java.lang.Math.PI/2) * Math.cos(east));
  }

  public int getY() {
      return (int)Math.round(EarthStep * Math.sin(north+java.lang.Math.PI/2) * Math.sin(east));
  }

  public int getZ() {
      return (int)Math.round(EarthStep * Math.cos(north-java.lang.Math.PI/2));
  }

  public String formatPolar(float n) { 
      DecimalFormat formatter = new DecimalFormat("000.0000");
      formatter.setNegativePrefix("-");
      formatter.setPositivePrefix("+");
      return formatter.format(n).replace(',','.');
  }

  
  public String formatMeters(float n) { 
      DecimalFormat formatter = new DecimalFormat("00000");
      formatter.setNegativePrefix("-");
      formatter.setPositivePrefix("+");
      return formatter.format(n).replace(',','.');
  }
  
  public static String formatCartesian(int n) { 

     if((n < minAllowed) || (n >maxAllowed))
     {
         throw new IllegalArgumentException("out of allowed Range");
     }
     char prefix;
     if(n < 0)
     {
          prefix = 'n';
          n = maxAllowed + n + 1 ;
     } else {
         prefix = 'p';
     }

     DecimalFormat fmt = new DecimalFormat(positionLength);
     return prefix +fmt.format(n).replace(',','.');
  }

  public static void main(String[] args) throws Exception {
      Domain2Position pos = new Domain2Position(0, 0);
            
      pos.setNorth(0.0f); pos.setEast (0.0f);
      System.out.println("Greenwich " + pos.getX() + " " + pos.getY() + " " + pos.getZ());
      
      pos.setNorth(90.0f); pos.setEast (0.0f);
      System.out.println("Northpole: " + pos.getX() + " " + pos.getY() + " " + pos.getZ());

      pos.setNorth(-90.0f); pos.setEast (0.0f);
      System.out.println("Southpole: " + pos.getX() + " " + pos.getY() + " " + pos.getZ());

      pos.setNorth(48.0f); pos.setEast (9.0f);
      System.out.println("Stuttgart: " + pos.getX() + " " + pos.getY() + " " + pos.getZ());

      pos.setNorth(0.0f); pos.setEast (0.0f);
      System.out.println("front: " + pos.getX() + " " + pos.getY() + " " + pos.getZ());

      pos.setNorth(0.0f); pos.setEast (90.0f);
      System.out.println("right: " + pos.getX() + " " + pos.getY() + " " + pos.getZ());

      pos.setNorth(0.0f); pos.setEast (-90.0f);
      System.out.println("left: " + pos.getX() + " " + pos.getY() + " " + pos.getZ());

      pos.setNorth(0.0f); pos.setEast (180.0f);
      System.out.println("back: " + pos.getX() + " " + pos.getY() + " " + pos.getZ());

  }
}
